//**************************************************************************************************
//                                        FrmHtmlVwr.hpp                                           *
//                                       ----------------                                          *
// Description : This is a class displays HTML documents ie. the gSpiceUI user manual.             *
//               operations.                                                                       *
// Started     : 2005-06-03                                                                        *
// Last Update : 2020-08-05                                                                        *
// Copyright   : (C) 2005-2020 MSWaters                                                            *
//**************************************************************************************************

//**************************************************************************************************
//                                                                                                 *
//      This program is free software; you can redistribute it and/or modify it under the          *
//      terms of the GNU General Public License as published by the Free Software Foundation;      *
//      either version 3 of the License, or (at your option) any later version.                    *
//                                                                                                 *
//**************************************************************************************************

#ifndef FRMHTMLVWR_HPP
#define FRMHTMLVWR_HPP

// Application includes

#include "TypeDefs.hpp"
#include "Config.hpp"

class FrmMain;

// wxWidgets includes

#include <wx/config.h>
#include <wx/filefn.h>
#include <wx/html/htmlwin.h>
#ifdef __WXMAC__
  #include <wx/image.h>
#endif

//**************************************************************************************************

class FrmHtmlVwr : public wxFrame
{
  private :

    FrmMain     * m_poFrmMain;       // Pointer to application main frame
    wxHtmlWindow  m_oHtmlWindow;     // The HTML viewer window
    wxString      m_rosInstallPath;  // The root path of the gSpiceUI installation

    // Object initialization functions
    void  Initialize     ( void );      // This function calls all other Init*( ) functions
    void  InitInstallPath( void );
    void  InitToolBar    ( void );
    void  InitHtmlWindow ( void );
    void  InitPosnSize   ( void );

  public :

    explicit  FrmHtmlVwr( FrmMain * poFrmMain );
             ~FrmHtmlVwr( );

    void  SetAppFrm( FrmMain * poFrmMain ) { m_poFrmMain = poFrmMain; }
    void  SetTitle ( void );

    void  DisplayUserMan( void );

    // Event handlers
    void  OnToolForwd( wxCommandEvent & roEvtCmd   );
    void  OnToolBckwd( wxCommandEvent & roEvtCmd   );
    void  OnToolClose( wxCommandEvent & roEvtCmd   );
    void  OnFrmClose ( wxCloseEvent   & roEvtClose );

    // In order to be able to react to a menu command, it must be given a
    // unique identifier such as a const or an enum.
    enum eFrmItemID
    {
      ID_TOOLBAR = 1,
      ID_TBR_FORWD,
      ID_TBR_BCKWD,
      ID_TBR_CLOSE,

      ID_UNUSED,      // Assigned to controls for which events are not used

      ID_FST = ID_TOOLBAR,
      ID_LST = ID_TBR_CLOSE
    };

    // Leave this as the last line as private access is envoked by this macro
#if wxCHECK_VERSION( 3,0,0 )
    wxDECLARE_EVENT_TABLE( );
#else
    wxDECLARE_EVENT_TABLE( )
#endif
};

//**************************************************************************************************

#endif // FRMHTMLVWR_HPP
