//**************************************************************************************************
//                                         PrcPdfVwr.cpp                                           *
//                                        ---------------                                          *
// Started     : 2020-08-03                                                                        *
// Last Update : 2020-08-05                                                                        *
// Copyright   : (C) 2020 MSWaters                                                                 *
//**************************************************************************************************

//**************************************************************************************************
//                                                                                                 *
//      This program is free software; you can redistribute it and/or modify it under the          *
//      terms of the GNU General Public License as published by the Free Software Foundation;      *
//      either version 3 of the License, or (at your option) any later version.                    *
//                                                                                                 *
//**************************************************************************************************

#include "PrcPdfVwr.hpp"

//**************************************************************************************************
// Allocate storage for static data members.

wxArrayString  PrcPdfVwr::m_osaPdfVwrBins;

//**************************************************************************************************
// Constructor.

PrcPdfVwr::PrcPdfVwr( void ) : PrcBase( wxPROCESS_REDIRECT )
{
  m_ofnNamePdf.Clear( );
}

//**************************************************************************************************
// Destructor.

PrcPdfVwr::~PrcPdfVwr( )
{
}

//**************************************************************************************************
// Load the PDF viewer utility list attribute m_osaPdfVwrBins with possible binary names.

void  PrcPdfVwr::InitPdfVwrBinLst( void )
{
  // Initialize the object attributes
  m_osaPdfVwrBins.Add( "evince" );
  m_osaPdfVwrBins.Add( "okular" );
  m_osaPdfVwrBins.Add( "zathura" );
  m_osaPdfVwrBins.Add( "acroread" );
  m_osaPdfVwrBins.Add( "qpdfview" );
  m_osaPdfVwrBins.Add( "atril" );
  m_osaPdfVwrBins.Add( "xpdf" );
  m_osaPdfVwrBins.Add( "mupdf" );
  m_osaPdfVwrBins.Add( "kpdf" );
  m_osaPdfVwrBins.Add( "epdf" );
  m_osaPdfVwrBins.Add( "gv" );
}

//**************************************************************************************************
// Find PDF viewer utility binaries on the system.
//
// Note : The list m_osaPdfVwrBins is reduced to only those binaries that are actually installed.
//
// Return Values:
//   true  - Success
//   false - Failure

bool  PrcPdfVwr::bFindPdfVwrBins( void )
{
  int         i1;
  wxFileName  ofn1;

  if( m_osaPdfVwrBins.IsEmpty( ) ) return( false );

  // Need to go thro. the list backwards because item positions change when an item is deleted
  for( i1=m_osaPdfVwrBins.GetCount( )-1; i1>=0; i1-- )
  {
    ofn1 = m_osaPdfVwrBins.Item( i1 );
    if( ! PrcBase::bFindBinFile( ofn1 ) ) m_osaPdfVwrBins.RemoveAt( i1 );
  }

  return( ! m_osaPdfVwrBins.IsEmpty( ) );
}

//**************************************************************************************************
// Set the PDF file name.
//
// Argument List:
//   psFileName - a string containing the full path and file name
//
// Return Values:
//   true  - Success
//   false - Failure

bool  PrcPdfVwr::bSetPdfFile( const wxString & rosFileName )
{
  wxFileName  ofn1;

  ofn1 = rosFileName;
  if( ofn1.GetPath( ).IsEmpty( ) ) ofn1.SetPath( wxT(".") );

  if( ! ofn1.IsOk( )       ) return( false );
  if( ! ofn1.FileExists( ) ) return( false );

  m_ofnNamePdf = ofn1;

  return( true );
}

//**************************************************************************************************
// View the PDF file.
//
// Return Values:
//   true  - Success
//   false - Failure

bool  PrcPdfVwr::bExec( void )
{
  wxString  os1;

  // Check that a binary file for the PDF viewer utility has been specified, if not try to find one
  if( ! bBinExists( ) ) return( false );

  // Check that a PDF file has been specified and is valid
  if( !m_ofnNamePdf.FileExists( ) || !m_ofnNamePdf.IsOk( ) )
  {
    os1 = m_ofnNamePdf.GetFullPath( );
    PrcBase::ClrErrMsg( );
    if( os1.IsEmpty( ) ) PrcBase::SetErrMsg( wxT("The PDF file name has not been set.") );
    else                 PrcBase::SetErrMsg( wxT("The PDF file doesn't exist : ") + os1 );
    return( false );
  }

  // Append the PDF file name to the argument list
  os1 = rosEscSpaceChrs( m_ofnNamePdf.GetFullPath( ) );
  if( ! bSetArgLst( os1 ) )      return( false );

  // Execute the process
  if( ! PrcBase::bExecAsync( ) ) return( false );

  return( true );
}

//**************************************************************************************************
